@@ -1308,12 +1308,10 @@ gnome_program_preinit (GnomeProgram *program,
     g_set_prgname (argv[0]);
 
     /* 0. Misc setup */
-    if (program->_priv->app_id)
-	g_free (program->_priv->app_id);
+    g_free (program->_priv->app_id);
     program->_priv->app_id = g_strdup (app_id);
     g_set_prgname (app_id);
-    if (program->_priv->app_version)
-	g_free (program->_priv->app_version);
+    g_free (program->_priv->app_version);
     program->_priv->app_version = g_strdup (app_version);
     program->_priv->argc = argc;
 
@@ -1739,6 +1737,10 @@ gnome_program_init (const char *app_id, const char *app_version,
     GnomeProgram *program;
     va_list args;
 
+    /* g_thread_init() has to be the first GLib function called ever */
+    if (!g_threads_got_initialized)
+        g_thread_init (NULL);
+
     g_type_init ();
 
     va_start(args, first_property_name);
